==> cvstrac-1.1.0.tar.gz: cvstrac-1.1.0/setup.c <==
/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** Implementation of the Setup page
*/
#include "config.h"
#include "setup.h"

/*
** WEBPAGE: /setup
*/
void setup_page(void){
  static const char *azLink[] = {
    "index", "Home", 0
  };
  char **azResult;
  int i;
  const char *zRoot, *zOldRoot;
  const char *zLog, *zOldLog;
  const char *zAsgnto, *zOldAsgnto;
  const char *zState, *zOldState;
  char *zSubsys;
  char *zHeader, *zFooter;
  const char *zNotify, *zOldNotify;

  /* The user must be at least the administrator in order to see
  ** this screen.
  */
  login_check_credentials();
  if( !g.okAdmin ){
    cgi_redirect("index");
    return;
  }

  /*
  ** If both "header" and "footer" query parameters are present, then
  ** change the header and footer to the values of those parameters.
  ** Only the setup user can do this.
  */
  if( g.okSetup && P("header") && P("footer") ){
    db_execute(
      "REPLACE INTO config VALUES('header','%q');"
      "REPLACE INTO config VALUES('footer','%q');",
       trim_string(P("header")),
       trim_string(P("footer"))
    );
  }

  /*
  ** The "r" query parameter is the name of the CVS repository root
  ** directory.  Change it if it has changed.
  */
  zOldRoot = db_short_query("SELECT value FROM config WHERE name='cvsroot'");
  if( zOldRoot==0 ) zOldRoot = "";
  zRoot = P("r");
  if( g.okSetup && zRoot && strcmp(zOldRoot,zRoot)!=0 ){
    db_execute("UPDATE config SET value='%q' WHERE name='cvsroot';", zRoot);
    zOldRoot = zRoot;
  }

  /*
  ** The "rrh" query parameter is present if the user presses the
  ** "Reread Revision History" button.  This causes the CVSROOT/history
  ** file to be reread.  Do this with caution as it erases any edits
  ** to the history that are currently in the database.  Only the
  ** setup user can do this.
  */
  if( g.okSetup && P("rrh") ){
    db_execute(
      "BEGIN;"
      "DELETE FROM chng WHERE not milestone;"
      "DELETE FROM filechng;"
      "DELETE FROM file;"
      "UPDATE config SET value=0 WHERE name='historysize';"
      "COMMIT;"
    );
    history_update();
  }

  /*
  ** Import users out of the CVSROOT/passwd file if the user pressed
  ** the Import Users button.  Only setup can do this.
  */
  if( g.okSetup && P("import_users") ){
    user_cvs_read();
  }

  /* 
  ** The "log" query parameter specifies a log file into which a record
  ** of all HTTP hits is written.  Write this value if this has changed.
  ** Only setup can make this change.
  */
  zOldLog = db_short_query("SELECT value FROM config WHERE name='logfile'");
  if( zOldLog==0 ) zOldLog = "";
  zLog = P("log");
  if( g.okSetup && zLog && strcmp(zOldLog,zLog)!=0 ){
    db_execute(
      "REPLACE INTO config(name,value) VALUES('logfile','%q');",
      zLog
    );
    zOldLog = zLog;
  }

  /*
  ** The "notify" query parameter is the name of a program or script that
  ** is run whenever a ticket is created or modified.  Modify the notify
  ** value if it has changed.  Only setup can do this.
  */
  zOldNotify = db_short_query("SELECT value FROM config WHERE name='notify'");
  if( zOldNotify==0 ) zOldNotify = "";
  zNotify = P("notify");
  if( g.okSetup && zNotify && strcmp(zOldNotify,zNotify)!=0 ){
    db_execute(
      "REPLACE INTO config(name,value) VALUES('notify','%q');",
      zNotify
    );
    zOldNotify = zNotify;
  }

  /*
  ** The "asgnto" query parameter specifies a userid who is assigned to
  ** all new tickets.  Record this value in the configuration table if
  ** it has changed.
  */
  zOldAsgnto = db_short_query("SELECT value FROM config WHERE name='assignto'");
  if( zOldAsgnto==0 ) zOldAsgnto = "";
  zAsgnto = P("asgnto");
  if( zAsgnto && strcmp(zOldAsgnto,zAsgnto)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='assignto';"
      "INSERT INTO config(name,value) VALUES('assignto','%q');"
      "COMMIT;",
      zAsgnto
    );
    zOldAsgnto = zAsgnto;
  }

  /*
  ** The "istate" query parameter specifies the initial state for new
  ** tickets.  Record any changes to this value.
  */
  zOldState = db_short_query("SELECT value FROM config "
                             "WHERE name='initial_state'");
  if( zOldState==0 ) zOldState = "";
  zState = P("istate");
  if( zState && strcmp(zOldState,zState)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='initial_state';"
      "INSERT INTO config(name,value) VALUES('initial_state','%q');"
      "COMMIT;",
      zState
    );
    zOldState = zState;
  }

  /*
  ** The "ss" query parameter is a long text string that specifies
  ** the names of all subsystems.  If any subsystem names have been
  ** added or removed, then make appropriate changes to the subsyst
  ** table in the database.
  */
  if( P("ss") ){
    zSubsys = mprintf("%s",P("ss"));
    db_execute("BEGIN; DELETE FROM subsyst;");
    while( zSubsys[0] ){
      while( isspace(zSubsys[0]) ){ zSubsys++; }
      if( zSubsys[0]==0 ) break;
      for(i=0; zSubsys[i] && !isspace(zSubsys[i]); i++){}
      if( zSubsys[i] ){
        zSubsys[i++] = 0;
      }
      db_execute("INSERT INTO subsyst(name) VALUES('%q')", zSubsys);
      zSubsys += i;
    }
    db_execute("COMMIT;");
  }

  /*
  ** Begin generating the Setup page
  */
  common_header("Configuration", azLink);
  if( g.okSetup ){
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Enter the full pathname of the root directory of the CVS repository:
    @ <br><input type="text" name="r" size="40" value="%h(zOldRoot)">
    @ <input type="submit" value="Submit"><br>
    @ After changing the CVS repository above, you will generally want to
    @ press the following button to cause the CVSROOT/history file to be
    @ reread from the new repository.
    @ <b>WARNING:</b> Pressing the button below will erase all file change
    @ history and reconstruct it from the CVSROOT/history file of the new
    @ CVS repository.  Any edits you may have made to existing file change
    @ comments will be lost.<br>
    @ <input type="submit" name="rrh" value="Reread CVSROOT/history">
    @ </form>
    @ </p>
    @ <hr>
    @
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Use the following button to automatically create a CVSTrac user ID
    @ for every user currently named in CVSROOT/passwd.  The new users will
    @ be given the same access permissions as user "anonymous" plus check-out
    @ permission and check-in permission if CVS allows the user to write.<br>
    @ <input type="submit" name="import_users" value="Import CVS Users">
    @ </form>
    @ </p>
    @ <hr>
    @
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Enter the name of a HTTP Logfile. Leave the name blank to disable logging:
    @ <br><input type="text" name="log" size="40" value="%h(zOldLog)">
    @ <input type="submit" value="Submit">
    @ </form>
    @ </p>
    @ <hr>
  }
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Select a user to whom new tickets will be assigned by default:<br>
  @ Assigned To:
  azResult = db_query("SELECT id FROM user UNION SELECT '' ORDER BY id");
  cgi_v_optionmenu(0, "asgnto", zOldAsgnto, (const char**)azResult);
  @ <input type="submit" value="Submit">
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Select the initial state that new tickets are created in:<br>
  @ Initial State:
  cgi_optionmenu(0, "istate", zOldState, "New", "new", "Review", "review", 
      "Defer", "defer", "Active", "active", "Fixed", "fixed", 
      "Tested", "tested", "Closed", "closed", 0);
  @ <input type="submit" value="Submit">
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ A whitespace-separated list of subsystems is shown below. 
  @ You may edit this text and press apply to change the list
  @ of available subsystems.
  @ </p>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  azResult = db_query("SELECT name FROM subsyst ORDER BY name");
  if( azResult[0]==0 ){
    @ <textarea cols=16 rows=10 name="ss"></textarea>
  }else{
    @ <textarea cols=16 rows=10 name="ss">%h(azResult[0])
    for(i=1; azResult[i]; i++){
      @ %h(azResult[i])
    }
    @ </textarea>
  }
  @ <br><input type="submit" value="Submit">
  @ </form>
  @ <hr>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Enter a shell command to run every time a ticket is
  @ created or modified.  The following substitutions are make
  @ on the string before it is passed to /bin/sh:
  @
  @ <blockquote>
  @ <dl compact>
  @ <dt><b>%%a</b></dt><dd>UserID of the person the ticket is assigned to</dd>
  @ <dt><b>%%c</b></dt><dd>Contact information for the originator</dd>
  @ <dt><b>%%d</b></dt><dd>The description</dd>
  @ <dt><b>%%n</b></dt><dd>The ticket number</dd>
  @ <dt><b>%%p</b></dt><dd>The project name</dd>
  @ <dt><b>%%r</b></dt><dd>The remarks section</dd>
  @ <dt><b>%%s</b></dt><dd>The status of the ticket</dd>
  @ <dt><b>%%t</b></dt><dd>The title of the ticket</dd>
  @ <dt><b>%%u</b></dt><dd>UserID of the person who made this change</dd>
  @ <dt><b>%%w</b></dt><dd>UserID of the originator of the ticket</dd>
  @ <dt><b>%%y</b></dt><dd>Type of ticket</dd>
  @ <dt><b>%%%%</b></dt><dd>The literal character "<b>%%</b>"</dd>
  @ </dl>
  @ </blockquote>
  @
  @ <input type="text" name="notify" size="70" value="%h(zOldNotify)">
  if( g.okSetup ){
    @ <input type="submit" value="Submit">
  }else{
    @ Only the "setup" user can change this value
  }
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ Enter HTML used for the header and footer of every page below.
  @ If you leave these entries blank, default headers and/or footers
  @ are used.  If you enter a filename (beginning with a "/" character)
  @ instead of HTML text, then the
  @ file is read at runtime and used for the header or footer.</p>
  @
  @ <p>Substitutions are made within the header and footer text.  These
  @ substitutions are made to the HTML regardless of whether the HTML
  @ is entered directly below or is read from a file.</p>
  @
  @ <blockquote>
  @ <dl compact>
  @ <dt><b>%%N</b></dt><dd>The name of the project</dd>
  @ <dt><b>%%T</b></dt><dd>The title of the current page</dd>
  @ <dt><b>%%V</b></dt><dd>The version number of CVSTrac</dd>
  @ <dt><b>%%%%</b></dt><dd>The literal character "<b>%%</b>"</dd>
  @ </dl>
  @ </blockquote>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  zHeader = db_short_query("SELECT value FROM config WHERE name='header'");
  zFooter = db_short_query("SELECT value FROM config WHERE name='footer'");
  @ Header:<br>
  @ <textarea cols=80 rows=10 name="header">%h(zHeader)</textarea><br>
  @ Footer:<br>
  @ <textarea cols=80 rows=10 name="footer">%h(zFooter)</textarea><br>
  if( g.okSetup ){
    @ <input type="submit" value="Submit">
  }else{
    @ (Only the "setup" user can change the header and footer text)
  }
  @ </form>
  @ </p>
  common_footer();
}

==> cvstrac-1.1.1.tar.gz: cvstrac-1.1.1/setup.c <==
/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** Implementation of the Setup page
*/
#include "config.h"
#include "setup.h"

/*
** WEBPAGE: /setup
*/
void setup_page(void){
  static const char *azLink[] = {
    "index", "Home", 0
  };
  char **azResult;
  int i;
  const char *zRoot, *zOldRoot;
  const char *zLog, *zOldLog;
  const char *zAsgnto, *zOldAsgnto;
  const char *zState, *zOldState;
  char *zSubsys;
  char *zHeader, *zFooter;
  const char *zNotify, *zOldNotify;

  /* The user must be at least the administrator in order to see
  ** this screen.
  */
  login_check_credentials();
  if( !g.okAdmin ){
    cgi_redirect("index");
    return;
  }

  /*
  ** If both "header" and "footer" query parameters are present, then
  ** change the header and footer to the values of those parameters.
  ** Only the setup user can do this.
  */
  if( g.okSetup && P("header") && P("footer") ){
    db_execute(
      "REPLACE INTO config VALUES('header','%q');"
      "REPLACE INTO config VALUES('footer','%q');",
       trim_string(P("header")),
       trim_string(P("footer"))
    );
  }

  /*
  ** The "r" query parameter is the name of the CVS repository root
  ** directory.  Change it if it has changed.
  */
  zOldRoot = db_short_query("SELECT value FROM config WHERE name='cvsroot'");
  if( zOldRoot==0 ) zOldRoot = "";
  zRoot = P("r");
  if( g.okSetup && zRoot && strcmp(zOldRoot,zRoot)!=0 ){
    db_execute("UPDATE config SET value='%q' WHERE name='cvsroot';", zRoot);
    zOldRoot = zRoot;
  }

  /*
  ** The "rrh" query parameter is present if the user presses the
  ** "Reread Revision History" button.  This causes the CVSROOT/history
  ** file to be reread.  Do this with caution as it erases any edits
  ** to the history that are currently in the database.  Only the
  ** setup user can do this.
  */
  if( g.okSetup && P("rrh") ){
    db_execute(
      "BEGIN;"
      "DELETE FROM chng WHERE not milestone;"
      "DELETE FROM filechng;"
      "DELETE FROM file;"
      "UPDATE config SET value=0 WHERE name='historysize';"
      "COMMIT;"
    );
    history_update();
  }

  /*
  ** Import users out of the CVSROOT/passwd file if the user pressed
  ** the Import Users button.  Only setup can do this.
  */
  if( g.okSetup && P("import_users") ){
    user_cvs_read();
  }

  /* 
  ** The "log" query parameter specifies a log file into which a record
  ** of all HTTP hits is written.  Write this value if this has changed.
  ** Only setup can make this change.
  */
  zOldLog = db_short_query("SELECT value FROM config WHERE name='logfile'");
  if( zOldLog==0 ) zOldLog = "";
  zLog = P("log");
  if( g.okSetup && zLog && strcmp(zOldLog,zLog)!=0 ){
    db_execute(
      "REPLACE INTO config(name,value) VALUES('logfile','%q');",
      zLog
    );
    zOldLog = zLog;
  }

  /*
  ** The "notify" query parameter is the name of a program or script that
  ** is run whenever a ticket is created or modified.  Modify the notify
  ** value if it has changed.  Only setup can do this.
  */
  zOldNotify = db_short_query("SELECT value FROM config WHERE name='notify'");
  if( zOldNotify==0 ) zOldNotify = "";
  zNotify = P("notify");
  if( g.okSetup && zNotify && strcmp(zOldNotify,zNotify)!=0 ){
    db_execute(
      "REPLACE INTO config(name,value) VALUES('notify','%q');",
      zNotify
    );
    zOldNotify = zNotify;
  }

  /*
  ** The "asgnto" query parameter specifies a userid who is assigned to
  ** all new tickets.  Record this value in the configuration table if
  ** it has changed.
  */
  zOldAsgnto = db_short_query("SELECT value FROM config WHERE name='assignto'");
  if( zOldAsgnto==0 ) zOldAsgnto = "";
  zAsgnto = P("asgnto");
  if( zAsgnto && strcmp(zOldAsgnto,zAsgnto)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='assignto';"
      "INSERT INTO config(name,value) VALUES('assignto','%q');"
      "COMMIT;",
      zAsgnto
    );
    zOldAsgnto = zAsgnto;
  }

  /*
  ** The "istate" query parameter specifies the initial state for new
  ** tickets.  Record any changes to this value.
  */
  zOldState = db_short_query("SELECT value FROM config "
                             "WHERE name='initial_state'");
  if( zOldState==0 ) zOldState = "";
  zState = P("istate");
  if( zState && strcmp(zOldState,zState)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='initial_state';"
      "INSERT INTO config(name,value) VALUES('initial_state','%q');"
      "COMMIT;",
      zState
    );
    zOldState = zState;
  }

  /*
  ** The "ss" query parameter is a long text string that specifies
  ** the names of all subsystems.  If any subsystem names have been
  ** added or removed, then make appropriate changes to the subsyst
  ** table in the database.
  */
  if( P("ss") ){
    zSubsys = mprintf("%s",P("ss"));
    db_execute("BEGIN; DELETE FROM subsyst;");
    while( zSubsys[0] ){
      while( isspace(zSubsys[0]) ){ zSubsys++; }
      if( zSubsys[0]==0 ) break;
      for(i=0; zSubsys[i] && !isspace(zSubsys[i]); i++){}
      if( zSubsys[i] ){
        zSubsys[i++] = 0;
      }
      db_execute("INSERT INTO subsyst(name) VALUES('%q')", zSubsys);
      zSubsys += i;
    }
    db_execute("COMMIT;");
  }

  /*
  ** Begin generating the Setup page
  */
  common_header("Configuration", azLink);
  if( g.okSetup ){
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Enter the full pathname of the root directory of the CVS repository:
    @ <br><input type="text" name="r" size="40" value="%h(zOldRoot)">
    @ <input type="submit" value="Submit"><br>
    @ After changing the CVS repository above, you will generally want to
    @ press the following button to cause the CVSROOT/history file to be
    @ reread from the new repository.
    @ <b>WARNING:</b> Pressing the button below will erase all file change
    @ history and reconstruct it from the CVSROOT/history file of the new
    @ CVS repository.  Any edits you may have made to existing file change
    @ comments will be lost.<br>
    @ <input type="submit" name="rrh" value="Reread CVSROOT/history">
    @ </form>
    @ </p>
    @ <hr>
    @
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Use the following button to automatically create a CVSTrac user ID
    @ for every user currently named in CVSROOT/passwd.  The new users will
    @ be given the same access permissions as user "anonymous" plus check-out
    @ permission and check-in permission if CVS allows the user to write.<br>
    @ <input type="submit" name="import_users" value="Import CVS Users">
    @ </form>
    @ </p>
    @ <hr>
    @
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Enter the name of a HTTP Logfile. Leave the name blank to disable logging:
    @ <br><input type="text" name="log" size="40" value="%h(zOldLog)">
    @ <input type="submit" value="Submit">
    @ </form>
    @ </p>
    @ <hr>
  }
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Select a user to whom new tickets will be assigned by default:<br>
  @ Assigned To:
  azResult = db_query("SELECT id FROM user UNION SELECT '' ORDER BY id");
  cgi_v_optionmenu(0, "asgnto", zOldAsgnto, (const char**)azResult);
  @ <input type="submit" value="Submit">
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Select the initial state that new tickets are created in:<br>
  @ Initial State:
  cgi_optionmenu(0, "istate", zOldState, "New", "new", "Review", "review", 
      "Defer", "defer", "Active", "active", "Fixed", "fixed", 
      "Tested", "tested", "Closed", "closed", 0);
  @ <input type="submit" value="Submit">
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ A whitespace-separated list of subsystems is shown below. 
  @ You may edit this text and press apply to change the list
  @ of available subsystems.
  @ </p>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  azResult = db_query("SELECT name FROM subsyst ORDER BY name");
  if( azResult[0]==0 ){
    @ <textarea cols=16 rows=10 name="ss"></textarea>
  }else{
    @ <textarea cols=16 rows=10 name="ss">%h(azResult[0])
    for(i=1; azResult[i]; i++){
      @ %h(azResult[i])
    }
    @ </textarea>
  }
  @ <br><input type="submit" value="Submit">
  @ </form>
  @ <hr>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Enter a shell command to run every time a ticket is
  @ created or modified.  The following substitutions are made
  @ on the string before it is passed to /bin/sh:
  @
  @ <table border=1 cellspacing=0 cellpadding=5 align="right" width="45%%">
  @ <tr><td bgcolor="#e0c0c0">
  @ <big><b>Important Security Note</b></big>
  @
  @ <p>Be sure to enclose all text substitutions in single-quotes.
  @ (ex <tt>'%%d'</tt>)  Otherwise, a user could cause arbitrary shell
  @ commands to be run on your system.</p>
  @  
  @ <p>Text is stripped of all single-quotes before it is substituted,
  @ so if the substitution is itself enclosed in single-quotes, it will always
  @ be treated as a single token by the shell.</p>
  @
  @ <p>For best security, use only the <b>%%n</b> substitution and have
  @ a Tcl or Perl script extract other fields directly from the database.</p>
  @ </td></tr></table>
  @
  @ <blockquote>
  @ <dl compact>
  @ <dt><b>%%a</b></dt><dd>UserID of the person the ticket is assigned to</dd>
  @ <dt><b>%%A</b></dt><dd>E-mail address of person assigned to</dd>
  @ <dt><b>%%c</b></dt><dd>Contact information for the originator</dd>
  @ <dt><b>%%d</b></dt><dd>The description</dd>
  @ <dt><b>%%n</b></dt><dd>The ticket number</dd>
  @ <dt><b>%%p</b></dt><dd>The project name</dd>
  @ <dt><b>%%r</b></dt><dd>The remarks section</dd>
  @ <dt><b>%%s</b></dt><dd>The status of the ticket</dd>
  @ <dt><b>%%t</b></dt><dd>The title of the ticket</dd>
  @ <dt><b>%%u</b></dt><dd>UserID of the person who made this change</dd>
  @ <dt><b>%%w</b></dt><dd>UserID of the originator of the ticket</dd>
  @ <dt><b>%%y</b></dt><dd>Type of ticket</dd>
  @ <dt><b>%%%%</b></dt><dd>The literal character "<b>%%</b>"</dd>
  @ </dl>
  @ </blockquote>
  @
  @ <input type="text" name="notify" size="70" value="%h(zOldNotify)">
  if( g.okSetup ){
    @ <input type="submit" value="Submit">
  }else{
    @ Only the "setup" user can change this value
  }
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ Enter HTML used for the header and footer of every page below.
  @ If you leave these entries blank, default headers and/or footers
  @ are used.  If you enter a filename (beginning with a "/" character)
  @ instead of HTML text, then the
  @ file is read at runtime and used for the header or footer.</p>
  @
  @ <p>Substitutions are made within the header and footer text.  These
  @ substitutions are made to the HTML regardless of whether the HTML
  @ is entered directly below or is read from a file.</p>
  @
  @ <blockquote>
  @ <dl compact>
  @ <dt><b>%%N</b></dt><dd>The name of the project</dd>
  @ <dt><b>%%T</b></dt><dd>The title of the current page</dd>
  @ <dt><b>%%V</b></dt><dd>The version number of CVSTrac</dd>
  @ <dt><b>%%%%</b></dt><dd>The literal character "<b>%%</b>"</dd>
  @ </dl>
  @ </blockquote>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  zHeader = db_short_query("SELECT value FROM config WHERE name='header'");
  zFooter = db_short_query("SELECT value FROM config WHERE name='footer'");
  @ Header:<br>
  @ <textarea cols=80 rows=10 name="header">%h(zHeader)</textarea><br>
  @ Footer:<br>
  @ <textarea cols=80 rows=10 name="footer">%h(zFooter)</textarea><br>
  if( g.okSetup ){
    @ <input type="submit" value="Submit">
  }else{
    @ (Only the "setup" user can change the header and footer text)
  }
  @ </form>
  @ </p>
  common_footer();
}
